=== modified file 'NEWS'
--- NEWS	2008-09-22 23:21:26 +0000
+++ NEWS	2008-09-23 05:37:53 +0000
@@ -28,6 +28,9 @@
 
     * Location aliases can now accept a trailing path.  (Micheal Hudson)
 
+    * New hooks ``Lock.hooks`` when LockDirs are acquired and released. 
+      (Robert Collins, MartinPool)
+
     * Switching in heavyweight checkouts uses the master branch's context, not
       the checkout's context.  (Adrian Wilkins)
 

=== modified file 'bzrlib/help_topics/en/hooks.txt'
--- bzrlib/help_topics/en/hooks.txt	2008-07-04 03:19:47 +0000
+++ bzrlib/help_topics/en/hooks.txt	2008-09-23 05:37:53 +0000
@@ -186,3 +186,16 @@
 
 Invoked whenever the server stops serving a directory.
 The hook signature is the same as ``server_started``.
+
+
+lock_acquired (LockDir)
+----------------------------
+
+Called with a LockResult object when a lock has been successfully acquired.
+(New in 1.8.)
+
+lock_released (LockDir)
+----------------------------
+
+Called with a LockResult object when a lock has been successfully released.
+(New in 1.8.)

=== modified file 'bzrlib/hooks.py'
--- bzrlib/hooks.py	2008-04-30 13:21:37 +0000
+++ bzrlib/hooks.py	2008-09-23 05:37:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007 Canonical Ltd
+# Copyright (C) 2007, 2008 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by

=== modified file 'bzrlib/lock.py'
--- bzrlib/lock.py	2008-04-24 07:22:53 +0000
+++ bzrlib/lock.py	2008-09-23 05:37:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006, 2007 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007, 2008 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -42,6 +42,42 @@
     osutils,
     trace,
     )
+from bzrlib.hooks import Hooks
+
+
+class LockHooks(Hooks):
+
+    def __init__(self):
+        Hooks.__init__(self)
+
+        # added in 1.8; called with a LockResult when a physical lock is
+        # acquired
+        self['lock_acquired'] = []
+
+        # added in 1.8; called with a LockResult when a physical lock is
+        # acquired
+        self['lock_released'] = []
+
+
+class Lock(object):
+    """Base class for locks.
+
+    :cvar hooks: Hook dictionary for operations on locks.
+    """
+
+    hooks = LockHooks()
+
+
+class LockResult(object):
+    """Result of an operation on a lock; passed to a hook"""
+
+    def __init__(self, lock_url, details=None):
+        """Create a lock result for lock with optional details about the lock."""
+        self.lock_url = lock_url
+        self.details = details
+
+    def __eq__(self, other):
+        return self.lock_url == other.lock_url and self.details == other.details
 
 
 class _OSLock(object):

=== modified file 'bzrlib/lockdir.py'
--- bzrlib/lockdir.py	2008-08-16 04:00:54 +0000
+++ bzrlib/lockdir.py	2008-09-23 05:37:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2007 Canonical Ltd
+# Copyright (C) 2006, 2007, 2008 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -110,6 +110,7 @@
 from bzrlib import (
     debug,
     errors,
+    lock,
     )
 import bzrlib.config
 from bzrlib.errors import (
@@ -126,6 +127,7 @@
         TransportError,
         UnlockableTransport,
         )
+from bzrlib.hooks import Hooks
 from bzrlib.trace import mutter, note
 from bzrlib.transport import Transport
 from bzrlib.osutils import rand_chars, format_delta, get_host_name
@@ -152,8 +154,9 @@
 _DEFAULT_POLL_SECONDS = 1.0
 
 
-class LockDir(object):
-    """Write-lock guarding access to data."""
+class LockDir(lock.Lock):
+    """Write-lock guarding access to data.
+    """
 
     __INFO_NAME = '/info'
 
@@ -296,6 +299,7 @@
             self._locked_via_token = False
             self._lock_held = False
         else:
+            old_nonce = self.nonce
             # rename before deleting, because we can't atomically remove the
             # whole tree
             start_time = time.time()
@@ -321,6 +325,10 @@
                 self.transport.delete_tree(tmpname)
             self._trace("... unlock succeeded after %dms",
                     (time.time() - start_time) * 1000)
+            result = lock.LockResult(self.transport.abspath(self.path),
+                old_nonce)
+            for hook in self.hooks['lock_released']:
+                hook(result)
 
     def break_lock(self):
         """Break a lock not held by this instance of LockDir.
@@ -450,7 +458,12 @@
         """
         if self._fake_read_lock:
             raise LockContention(self)
-        return self._attempt_lock()
+        result = self._attempt_lock()
+        hook_result = lock.LockResult(self.transport.abspath(self.path),
+                self.nonce)
+        for hook in self.hooks['lock_acquired']:
+            hook(hook_result)
+        return result
 
     def wait_lock(self, timeout=None, poll=None, max_attempts=None):
         """Wait a certain period for a lock.

=== modified file 'bzrlib/tests/test_lockdir.py'
--- bzrlib/tests/test_lockdir.py	2008-06-17 04:07:49 +0000
+++ bzrlib/tests/test_lockdir.py	2008-09-23 05:37:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2007 Canonical Ltd
+# Copyright (C) 2006, 2007, 2008 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@
 from bzrlib import (
     config,
     errors,
+    lock,
     osutils,
     tests,
     transport,
@@ -658,3 +659,74 @@
         self.assertRaises(errors.LockContention, ld2.attempt_lock)
         # no kibble
         check_dir(['held'])
+
+    def record_hook(self, result):
+        self._calls.append(result)
+
+    def reset_hooks(self):
+        self._old_hooks = lock.Lock.hooks
+        self.addCleanup(self.restore_hooks)
+        lock.Lock.hooks = lock.LockHooks()
+
+    def restore_hooks(self):
+        lock.Lock.hooks = self._old_hooks
+
+    def test_LockDir_acquired_success(self):
+        # the LockDir.lock_acquired hook fires when a lock is acquired.
+        self._calls = []
+        self.reset_hooks()
+        LockDir.hooks.install_named_hook('lock_acquired',
+            self.record_hook, 'record_hook')
+        ld = self.get_lock()
+        ld.create()
+        self.assertEqual([], self._calls)
+        result = ld.attempt_lock()
+        lock_path = ld.transport.abspath(ld.path)
+        self.assertEqual([lock.LockResult(lock_path, result)], self._calls)
+        ld.unlock()
+        self.assertEqual([lock.LockResult(lock_path, result)], self._calls)
+
+    def test_LockDir_acquired_fail(self):
+        # the LockDir.lock_acquired hook does not fire on failure.
+        self._calls = []
+        self.reset_hooks()
+        ld = self.get_lock()
+        ld.create()
+        ld2 = self.get_lock()
+        ld2.attempt_lock()
+        # install a lock hook now, when the disk lock is locked
+        LockDir.hooks.install_named_hook('lock_acquired',
+            self.record_hook, 'record_hook')
+        self.assertRaises(errors.LockContention, ld.attempt_lock)
+        self.assertEqual([], self._calls)
+        ld2.unlock()
+        self.assertEqual([], self._calls)
+
+    def test_LockDir_released_success(self):
+        # the LockDir.lock_released hook fires when a lock is acquired.
+        self._calls = []
+        self.reset_hooks()
+        LockDir.hooks.install_named_hook('lock_released',
+            self.record_hook, 'record_hook')
+        ld = self.get_lock()
+        ld.create()
+        self.assertEqual([], self._calls)
+        result = ld.attempt_lock()
+        self.assertEqual([], self._calls)
+        ld.unlock()
+        lock_path = ld.transport.abspath(ld.path)
+        self.assertEqual([lock.LockResult(lock_path, result)], self._calls)
+
+    def test_LockDir_released_fail(self):
+        # the LockDir.lock_released hook does not fire on failure.
+        self._calls = []
+        self.reset_hooks()
+        ld = self.get_lock()
+        ld.create()
+        ld2 = self.get_lock()
+        ld.attempt_lock()
+        ld2.force_break(ld2.peek())
+        LockDir.hooks.install_named_hook('lock_released',
+            self.record_hook, 'record_hook')
+        self.assertRaises(LockBroken, ld.unlock)
+        self.assertEqual([], self._calls)

